@@ -29,7 +29,8 @@
 #ifndef __CDIALOG_H
 #define __CDIALOG_H
 
-#include "gambas.h"
+#include "main.h"
+#include "gdialog.h"
 
 #ifndef __CDIALOG_CPP
 extern GB_DESC CDialogDesc[];
