@@ -71,17 +71,31 @@ static void *CLASS_UserControl = NULL;
 static bool has_action(void *control)
 {
 	if (GB.Is(control, GB.FindClass("Menu")))
-		return ((CMENU *)(control))->widget->action();
+	{
+		gMenu *menu = ((CMENU *)(control))->widget;
+		return menu ? menu->action() : false;
+	}
 	else
-		return ((CWIDGET *)(control))->widget->action();
+	{
+		gControl *ctrl = ((CWIDGET *)(control))->widget;
+		return ctrl ? ctrl->action() : false;
+	}
 }
 
 static void set_action(void *control, bool v)
 {
 	if (GB.Is(control, GB.FindClass("Menu")))
-		((CMENU *)(control))->widget->setAction(v);
+	{
+		gMenu *menu = ((CMENU *)(control))->widget;
+		if (menu)
+			menu->setAction(v);
+	}
 	else
-		((CWIDGET *)(control))->widget->setAction(v);
+	{
+		gControl *ctrl = ((CWIDGET *)(control))->widget;
+		if (ctrl)
+			ctrl->setAction(v);
+	}
 }
 
 #define HAS_ACTION(_control) has_action(_control)
@@ -108,6 +122,14 @@ void gb_plug_raise_unplugged(gControl *sender)
 	GB.Raise((void*)_ob,EVENT_Unplugged,0);
 }
 
+void gb_plug_raise_error(gControl *sender)
+{
+	CWIDGET *_ob=GetObject(sender);
+	
+	if (!_ob) return;
+	GB.Raise((void*)_ob,EVENT_PlugError,0);
+}
+
 // static void raise_menu(void *_object)
 // {
 // 	GB.Raise(THIS, EVENT_Menu, 0);
@@ -306,8 +328,7 @@ CWIDGET *GetContainer(CWIDGET *control)
 
 int CWIDGET_check(void *_object)
 {
-	if (!CONTROL) return true;
-  	return false;
+	return (!CONTROL || CONTROL->isDestroyed());
 }
 
 /*************************************************************************************
@@ -319,41 +340,30 @@ Embedder
 BEGIN_METHOD(CPLUGIN_new, GB_OBJECT parent)
 
 	InitControl(new gPlugin(CONTAINER(VARG(parent))), (CWIDGET*)_object);
-	((gPlugin*)((CPLUGIN*)_object)->widget)->onPlug=gb_plug_raise_plugged;
-	((gPlugin*)((CPLUGIN*)_object)->widget)->onUnplug=gb_plug_raise_unplugged;
+	PLUGIN->onPlug = gb_plug_raise_plugged;
+	PLUGIN->onUnplug = gb_plug_raise_unplugged;
+	PLUGIN->onError = gb_plug_raise_error;
 
 END_METHOD
 
 BEGIN_PROPERTY(CPLUGIN_client)
 
-	GB.ReturnPointer((void *)((gPlugin*)((CPLUGIN*)_object)->widget)->client() );
+	GB.ReturnPointer((void *)PLUGIN->client());
 
 END_PROPERTY
 
 BEGIN_METHOD(CPLUGIN_embed, GB_POINTER id; GB_BOOLEAN prepared)
 
-	((gPlugin*)((CPLUGIN*)_object)->widget)->plug(VARG(id), VARGOPT(prepared, TRUE));
+	PLUGIN->plug(VARG(id), VARGOPT(prepared, FALSE));
 
 END_METHOD
 
 BEGIN_METHOD_VOID(CPLUGIN_discard)
 
-	((gPlugin*)((CPLUGIN*)_object)->widget)->discard();
+	PLUGIN->discard();
 
 END_METHOD
 
-// BEGIN_PROPERTY(CPLUGIN_border)
-// 
-//   	if (READ_PROPERTY) 
-// 	{
-//   		GB.ReturnInteger(((gPlugin*)((CPLUGIN*)_object)->widget)->getBorder());
-// 		return;
-//    	}
-// 	
-// 	((gPlugin*)((CPLUGIN*)_object)->widget)->setBorder(VPROP(GB_INTEGER));
-// 
-// END_PROPERTY
-
 
 /**************************************************************************************
 
@@ -411,7 +421,7 @@ BEGIN_PROPERTY(CCONTROL_font)
 
 	if (!THIS->font)
 	{
-		THIS->font = CFONT_create(CONTROL->font()->copy(), 0, THIS);
+		THIS->font = CFONT_create(CONTROL->font(), 0, THIS);
 		GB.Ref(THIS->font);
 	}
 	
@@ -425,34 +435,15 @@ BEGIN_PROPERTY(CCONTROL_font)
     if (font)
     {
     	CONTROL->setFont(font->font);
-			if (font != THIS->font)
-			{
-				GB.Unref(POINTER(&THIS->font));
-				THIS->font = NULL;
-			}
-    }
+			if (font == THIS->font)
+				return;
+		}
+		GB.Unref(POINTER(&THIS->font));
+		THIS->font = NULL;
 	}
 
 END_PROPERTY
 
-#if 0
-BEGIN_PROPERTY(CCONTROL_font)
-
-  CFONT *font;
-
-  if (READ_PROPERTY)
-  {
-    GB.ReturnObject(CFONT_create(CONTROL->font(), 0, THIS));
-  }
-  else
-  {
-    font = (CFONT *)VPROP(GB_OBJECT);
-    CONTROL->setFont(font ? font->font : 0);
-  }
-
-END_PROPERTY
-#endif
-
 BEGIN_PROPERTY(CWIDGET_design)
 
 	if (READ_PROPERTY) { GB.ReturnBoolean(CONTROL->design()); return; }
@@ -889,11 +880,10 @@ GB_DESC CPluginDesc[] =
   GB_METHOD("Discard", 0, CPLUGIN_discard, 0),
   
   GB_PROPERTY_READ("Client", "p", CPLUGIN_client),
-  //GB_PROPERTY("Border", "i<Border>", CPLUGIN_border),
   
   GB_EVENT("Embed", 0, 0, &EVENT_Plugged),
   GB_EVENT("Close", 0, 0, &EVENT_Unplugged),
-  GB_EVENT("Error", 0, 0, &EVENT_PlugError), /* TODO */
+  GB_EVENT("Error", 0, 0, &EVENT_PlugError),
   
   EMBEDDER_DESCRIPTION,
   
