@@ -87,27 +87,31 @@ static void gb_raise_window_Open(gMainWindow *sender)
 
 static void gb_raise_window_Show(gMainWindow *sender)
 {
-	CWIDGET *_ob=GetObject(sender);
+	CWIDGET *ob = GetObject(sender);
 
-	if (!_ob) return;
-	GB.Raise((void*)_ob,EVENT_Show,0);
+	if (!ob) return;
+	GB.Ref(ob);
+	GB.Raise((void*)ob,EVENT_Show,0);
 	if (!sender->spontaneous())
-		CACTION_raise(_ob);
+		CACTION_raise(ob);
+	GB.Unref(POINTER(&ob));
 }
 
-static void gb_post_window_Show(gMainWindow *sender)
+/*static void gb_post_window_Show(gMainWindow *sender)
 {
 	GB.Post( (void (*)())gb_raise_window_Show,(long)sender);
-}
+}*/
 
 static void gb_raise_window_Hide(gMainWindow *sender)
 {
-	CWIDGET *_ob=GetObject(sender);
+	CWIDGET *ob=GetObject(sender);
 
-	if (!_ob) return;
-	GB.Raise((void*)_ob,EVENT_Hide,0);
+	if (!ob) return;
+	GB.Ref(ob);
+	GB.Raise((void*)ob,EVENT_Hide,0);
 	if (!sender->spontaneous())
-		CACTION_raise(_ob);
+		CACTION_raise(ob);
+	GB.Unref(POINTER(&ob));
 }
 
 static void gb_raise_window_Move(gMainWindow *sender)
@@ -188,16 +192,12 @@ static bool gb_raise_window_Close(gMainWindow *sender)
     if (closeAll())
       return true;
     if (!sender->isPersistent())
+    {
     	deleteAll();
+    	MAIN_Window = NULL;
+		}
   }
 
-	// BM: isn't it already done?
-	/*if (sender->isPersistent()) 
-	{
-		sender->hide();
-		return true;
-	}*/
-
 	if (_ob->embed)
 	{
 		CWINDOW_Embedder = 0;
@@ -300,7 +300,7 @@ BEGIN_METHOD(CWINDOW_new, GB_OBJECT parent;)
 
 	if ( (!MAIN_Window) && (!parent)) 
 	{
-		MAIN_Window=THIS;
+		MAIN_Window = THIS;
 		//fprintf(stderr, "MAIN_Window = %p\n", MAIN_Window);
 	}
 
@@ -316,49 +316,6 @@ BEGIN_METHOD_VOID(CWINDOW_free)
 
 END_METHOD
 
-BEGIN_METHOD_VOID(CFORM_new)
-
-	GB.Attach(_object, _object, "Form");
-
-END_METHOD
-
-
-BEGIN_METHOD_VOID(CFORM_main)
-
-	CWINDOW *form;
-
-	form = (CWINDOW *)GB.AutoCreate(GB.GetClass(NULL), 0);
-	form->ob.widget->show();
-
-END_METHOD
-
-
-/*BEGIN_METHOD(CFORM_load, GB_OBJECT parent;GB_BOOLEAN Plug;)
-
-	void *Par=NULL;
-	bool Plg=false;
-	long npar=0;
-
-	if (!MISSING(parent)) { Par=VARG(parent); npar++; }
-	if (!MISSING(Plug)) { Plg=VARG(Plug); npar++; }
-
-	GB.Push(1, GB_T_OBJECT,Par);
-	GB.AutoCreate(GB.GetClass(NULL),npar);
-
-END_METHOD*/
-
-BEGIN_METHOD(CFORM_load, GB_OBJECT parent)
-
-  if (!MISSING(parent))
-  {
-    GB.Push(1, GB_T_OBJECT, VARG(parent));
-    //qDebug("CFORM_load + parent");
-  }
-
-  GB.AutoCreate(GB.GetClass(NULL), MISSING(parent) ? 0 : 1);
-
-END_METHOD
-
 
 BEGIN_METHOD_VOID(CWINDOW_next)
 
@@ -659,6 +616,35 @@ BEGIN_PROPERTY(CWINDOW_closed)
 
 END_PROPERTY
 
+BEGIN_METHOD_VOID(CFORM_new)
+
+	GB.Attach(_object, _object, "Form");
+
+END_METHOD
+
+
+BEGIN_METHOD_VOID(CFORM_main)
+
+	CWINDOW *form;
+
+	form = (CWINDOW *)GB.AutoCreate(GB.GetClass(NULL), 0);
+	if (!((gMainWindow *)form->ob.widget)->isHidden())
+		CWINDOW_show(form, NULL);
+
+END_METHOD
+
+
+BEGIN_METHOD(CFORM_load, GB_OBJECT parent)
+
+  if (!MISSING(parent))
+  {
+    GB.Push(1, GB_T_OBJECT, VARG(parent));
+    //qDebug("CFORM_load + parent");
+  }
+
+  GB.AutoCreate(GB.GetClass(NULL), MISSING(parent) ? 0 : 1);
+
+END_METHOD
 
 
 /***************************************************************************
