@@ -26,6 +26,8 @@
 
 ***************************************************************************/
 
+#include <unistd.h>
+
 #ifndef GAMBAS_DIRECTFB
 #ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
@@ -117,19 +119,38 @@ gPlugin::gPlugin(gContainer *parent) : gControl(parent)
 
 long gPlugin::client()
 {
-	GdkNativeWindow win=gtk_socket_get_id(GTK_SOCKET(widget));
-	return (long)win;
+	//GdkNativeWindow win = gtk_socket_get_id(GTK_SOCKET(widget));
+	//return (long)win;
+	GdkWindow *win = GTK_SOCKET(widget)->plug_window;
+	if (!win)
+		return 0;
+	else
+		return (long)GDK_WINDOW_XID(win);
 }
 
-void gPlugin::plug(long id,bool prepared)
+void gPlugin::plug(long id, bool prepared)
 {
-	//int bc;
+	void (*func)(gControl *);
+	int i;
 
-	//for (bc=0; bc<10; bc++)
-	//{
-		if (!prepared) gtk_socket_steal(GTK_SOCKET(widget),(GdkNativeWindow)id);
-		else gtk_socket_add_id(GTK_SOCKET(widget),(GdkNativeWindow)id);
-	//}
+	func = onPlug;
+	onPlug = NULL;
+
+	for (i = 1; i >= 0; i--)
+	{
+		if (i == 0)
+			onPlug = func;
+			
+		if (!prepared) 
+			gtk_socket_steal(GTK_SOCKET(widget), (GdkNativeWindow)id);
+		else 
+			gtk_socket_add_id(GTK_SOCKET(widget), (GdkNativeWindow)id);
+	}
+	
+	if (client())
+    XAddToSaveSet(gdk_display, client());
+	else
+		emit(SIGNAL(onError));
 }
 
 void gPlugin::discard()
@@ -138,24 +159,14 @@ void gPlugin::discard()
 	stub("DIRECTFB/gPlugin:discard()");
 	#else
 	#ifdef GDK_WINDOWING_X11
-	GtkWidget *fr;
-	gColor color;
 	
-	GdkNativeWindow win=gtk_socket_get_id(GTK_SOCKET(widget));
-	GdkDisplay* dsp=gdk_display_get_default();
+	//gColor color;
 	
-	if (!win) return;
+	if (!client()) return;
 	
-	XReparentWindow(GDK_DISPLAY_XDISPLAY(dsp),win,GDK_ROOT_WINDOW(), 0, 0);
+  XRemoveFromSaveSet(gdk_display, client());
+	XReparentWindow(gdk_display, client(), GDK_ROOT_WINDOW(), 0, 0);
 	
-	color = foreground();
-	gtk_widget_destroy(widget);
-	
-	border=widget=gtk_socket_new();
-	fr=gtk_bin_get_child(GTK_BIN(border));
-	gtk_container_add(GTK_CONTAINER(fr),widget);
-	gtk_widget_show(widget);
-	setForeground(color);
 	#else
 	stub("no-X11/gPlugin:discard()");
 	#endif
@@ -168,6 +179,7 @@ void gPlugin::discard()
 CREATION AND DESTRUCTION
 
 ******************************************************************/
+
 void gControl::cleanRemovedControls()
 {
 	GList *iter;
@@ -1131,7 +1143,7 @@ static gboolean cb_frame_expose(GtkWidget *wid, GdkEventExpose *e, gControl *con
 	return false;
 }
 
-static void cb_size_allocate(GtkWidget *wid, GdkEventExpose *e, gContainer *container)
+static void cb_size_allocate(GtkWidget *wid, GtkAllocation *a, gContainer *container)
 {
 	if (!container->isTopLevel())
 		container->performArrange();
@@ -1255,7 +1267,13 @@ int gControl::getFrameWidth()
 			return 2;
   }
   
-  return 0;
+  switch (frame_border)
+  {
+    case BORDER_NONE: p = 0; break;
+    case BORDER_PLAIN: p = 1; break;
+    default: p = 2; break; // TODO: Use style
+  }
+  return p;
 }
 
 void gControl::setFrameBorder(int border)
@@ -1289,7 +1307,7 @@ gColor gControl::realBackground()
 {
 	if (bg_set)
 		return use_base ? get_gdk_base_color(widget) : get_gdk_bg_color(widget);
-	else if (!isWindow())
+	else if (pr)
 		return pr->realBackground();
 	else
 		return COLOR_DEFAULT;
@@ -1326,9 +1344,8 @@ void gControl::setBackground(gColor color)
 	
 	if (!bg_set)
 	{
-		if (isWindow())
-			return;
-		color = pr->realBackground();
+		if (pr)
+			color = pr->realBackground();
 	}
 			
 	setRealBackground(color);
@@ -1338,7 +1355,7 @@ gColor gControl::realForeground()
 {
 	if (fg_set)
 		return use_base ? get_gdk_text_color(widget) : get_gdk_fg_color(widget);
-	else if (!isWindow())
+	else if (pr)
 		return pr->realForeground();
 	else
 		return COLOR_DEFAULT;
@@ -1375,9 +1392,8 @@ void gControl::setForeground(gColor color)
 	
 	if (!fg_set)
 	{
-		if (isWindow())
-			return;
-		color = pr->realForeground();
+		if (pr)
+			color = pr->realForeground();
 	}
 	
 	setRealForeground(color);
