@@ -28,6 +28,7 @@
 
 #include <ctype.h>
 #include <time.h>
+#include <unistd.h>
 
 #include "widgets.h"
 #include "widgets_private.h"
@@ -53,6 +54,7 @@ static gboolean win_frame(GtkWidget *widget,GdkEventWindowState *event,gMainWind
 static void cb_show (GtkWidget *widget, gMainWindow *data)
 {
 	//fprintf(stderr, "cb_show: %p\n", data);
+	//data->afterShow();
 	data->emitOpen();
 	if (data->opened)
 	{
@@ -121,26 +123,34 @@ static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainW
 			//g_debug("gMainWindow: cb_configure: %d", event->send_event);
 			data->performArrange();
 			if (data->onResize) data->onResize(data);*/
-			if (data->_resized || data->parent())
+			
+			//if (data->_resized || data->parent())
 			{
 				data->_resized = false;
 				resize_later(data);
 			}
-			else
-			{
-				//resize_later(data);
-				if (!data->_next_timer)
-					data->_next_timer = g_timeout_add(50, (GSourceFunc)resize_later, data);
-			}
+			//else
+			//{
+			//	//resize_later(data);
+			//	if (!data->_next_timer)
+			//		data->_next_timer = g_timeout_add(50, (GSourceFunc)resize_later, data);
+			//}
 		}
 	}
 	
 	return false;
 }
 
-static void cb_open(GtkWidget *widget, GdkEvent *e, gMainWindow *data)
+static gboolean cb_open(GtkWidget *widget, GdkEvent *e, gMainWindow *data)
 {
 	data->emitOpen();
+	return FALSE;
+}
+
+static gboolean cb_map(GtkWidget *widget, GdkEvent *e, gMainWindow *data)
+{
+	data->afterShow();
+	return FALSE;
 }
 
 static gboolean cb_expose(GtkWidget *wid, GdkEventExpose *e, gMainWindow *data)
@@ -191,6 +201,8 @@ void gMainWindow::initialize()
 	_style = NULL;
 	_next_timer = 0;
 	_xembed = false;
+	_activate = false;
+	_hidden = false;
 
 	onOpen = NULL;
 	onShow = NULL;
@@ -207,7 +219,7 @@ void gMainWindow::initWindow()
 {
 	//resize(200,150);
 	
-	if (parent())
+	if (!isTopLevel())
 	{
 		g_signal_connect(G_OBJECT(border), "show", G_CALLBACK(cb_show), (gpointer)this);
 		g_signal_connect(G_OBJECT(border), "configure-event", G_CALLBACK(cb_configure), (gpointer)this);
@@ -215,6 +227,7 @@ void gMainWindow::initWindow()
 		g_signal_connect(G_OBJECT(border),"show",G_CALLBACK(cb_show),(gpointer)this);
 		g_signal_connect(G_OBJECT(border),"hide",G_CALLBACK(cb_hide),(gpointer)this);
 		//g_signal_connect_after(G_OBJECT(border), "size-allocate", G_CALLBACK(cb_configure), (gpointer)this);
+		g_signal_connect(G_OBJECT(widget), "expose-event", G_CALLBACK(cb_expose), (gpointer)this);
 	}
 	else
 	{
@@ -224,6 +237,7 @@ void gMainWindow::initWindow()
 		g_signal_connect(G_OBJECT(border), "configure-event",G_CALLBACK(cb_configure),(gpointer)this);
 		g_signal_connect(G_OBJECT(border), "delete-event",G_CALLBACK(win_close),(gpointer)this);
 		g_signal_connect(G_OBJECT(border), "window-state-event",G_CALLBACK(win_frame),(gpointer)this);
+		g_signal_connect_after(G_OBJECT(border), "map-event", G_CALLBACK(cb_map), (gpointer)this);
 		
 		gtk_widget_add_events(widget,GDK_BUTTON_MOTION_MASK);
 		g_signal_connect(G_OBJECT(widget), "expose-event", G_CALLBACK(cb_expose), (gpointer)this);
@@ -271,8 +285,8 @@ gMainWindow::gMainWindow(gContainer *par) : gContainer(par)
 	initialize();
 	g_typ = Type_gMainWindow;
 	
-	border=gtk_event_box_new();
-	widget=gtk_layout_new(0,0);
+	border = gtk_event_box_new();
+	widget = gtk_layout_new(0,0);
 	
 	realize(false);
 	
@@ -287,6 +301,8 @@ gMainWindow::~gMainWindow()
 	{
 		emit(SIGNAL(onClose));
 		opened = false;
+		if (GTK_IS_WINDOW(border) && modal())
+			gApplication::exitLoop();
 	}
 	
 	if (_next_timer)
@@ -315,7 +331,7 @@ int gMainWindow::getStacking()
 void gMainWindow::setSticky(bool vl)
 {
 	sticky=vl;
-	if (parent()) return;
+	if (!isTopLevel()) return;
 
 	if (vl) gtk_window_stick(GTK_WINDOW(border));
 	else    gtk_window_unstick(GTK_WINDOW(border));
@@ -324,7 +340,7 @@ void gMainWindow::setSticky(bool vl)
 void gMainWindow::setStacking(int vl)
 {
   stack=vl;
-	if (parent()) return;
+	if (!isTopLevel()) return;
 
 	switch (vl)
 	{
@@ -410,6 +426,15 @@ void gMainWindow::emitOpen()
 	}
 }
 
+void gMainWindow::afterShow()
+{
+	if (_activate)
+	{
+		gtk_window_present(GTK_WINDOW(border));
+		_activate = false;
+	}
+}
+
 void gMainWindow::setVisible(bool vl)
 {
 	GtkWindowGroup *group;
@@ -432,13 +457,14 @@ void gMainWindow::setVisible(bool vl)
 			
 		_not_spontaneous = !visible;
 		visible = true;
+		_hidden = false;
 		
 		if (isTopLevel())
 		{
 			if (!_title || !*_title)
 				gtk_window_set_title(GTK_WINDOW(border), gApplication::defaultTitle());
 			gtk_window_present(GTK_WINDOW(border));
-			drawMask();
+			gtk_window_move(GTK_WINDOW(border), bufX, bufY);
 		}
 		else 
 		{
@@ -446,18 +472,16 @@ void gMainWindow::setVisible(bool vl)
 			parent()->performArrange();
 		}
 		
-		/*if (!focus)
-		{
-			focus = findFirstFocus();
-			//fprintf(stderr, "findFirstFocus: %s\n", focus ? focus->name() : NULL); 
-		}*/
+		drawMask();
 		
 		if (focus)
 		{
-			//fprintf(stderr, "focus = %s\n", focus->name());
 			focus->setFocus();
 			focus = 0;
 		}
+		
+		if (skipTaskBar())
+			_activate = true;
 	}
 	else
 	{
@@ -465,6 +489,7 @@ void gMainWindow::setVisible(bool vl)
 			focus = gDesktop::activeControl();
 			
 		_not_spontaneous = visible;
+		_hidden = true;
 		gControl::setVisible(false);
 	}
 }
@@ -507,7 +532,7 @@ void gMainWindow::center()
 {
 	int myx,myy;
 	
-	if (parent()) return;
+	if (!isTopLevel()) return;
 	
 	myx=(gDesktop::width()/2)-(width()/2);
 	myy=(gDesktop::height()/2)-(height()/2);
@@ -517,7 +542,7 @@ void gMainWindow::center()
 
 bool gMainWindow::modal()
 {
-	if (parent()) return false;
+	if (!isTopLevel()) return false;
 
 	return gtk_window_get_modal (GTK_WINDOW(border));
 }
@@ -547,11 +572,7 @@ void gMainWindow::showModal()
 
 	//fprintf(stderr, "showModal: begin %p\n", this);
 
-	do
-	{
-		do_iteration(false);
-	}
-	while (opened);
+	gApplication::enterLoop();
 	
 	//fprintf(stderr, "showModal: end %p\n", this);
 
@@ -559,6 +580,8 @@ void gMainWindow::showModal()
 	_current_group = save_group;	
 	_current = save;
 	
+	gtk_window_set_modal(GTK_WINDOW(border), false);
+	
 	if (!persistent)
 		destroyNow();
 	else
@@ -568,7 +591,7 @@ void gMainWindow::showModal()
 
 void gMainWindow::raise()
 {
-	if (parent()) { gControl::raise(); return; }
+	if (!isTopLevel()) { gControl::raise(); return; }
 	gtk_window_present(GTK_WINDOW(border));
 }
 
@@ -579,7 +602,7 @@ const char* gMainWindow::text()
 
 bool gMainWindow::skipTaskBar()
 {
-	if (parent()) 
+	if (!isTopLevel()) 
 		return false;
 	else
 		return _skip_taskbar;
@@ -597,7 +620,7 @@ void gMainWindow::setText(const char *txt)
 
 int gMainWindow::getBorder()
 {
-	if (parent()) return 0;
+	if (!isTopLevel()) return 0;
 	
 	if (!gtk_window_get_decorated(GTK_WINDOW(border))) return 0;
 	if (!gtk_window_get_resizable(GTK_WINDOW(border))) return 1;
@@ -606,7 +629,7 @@ int gMainWindow::getBorder()
 
 void gMainWindow::setBorder(int b)
 {
-  if (parent()) return;
+  if (!isTopLevel()) return;
 	 
 	switch (b)
 	{
@@ -635,7 +658,7 @@ void gMainWindow::setBorder(int b)
 void gMainWindow::setSkipTaskBar(bool b)
 {
 	_skip_taskbar = b;
-	if (parent()) return;
+	if (!isTopLevel()) return;
 	gtk_window_set_skip_taskbar_hint (GTK_WINDOW(border), b);
 }
 
@@ -659,20 +682,20 @@ void gMainWindow::setIcon(gPicture *pic)
 {
   gPicture::assign(&_icon, pic);
 
-	if (parent()) return;
+	if (!isTopLevel()) return;
   gtk_window_set_icon(GTK_WINDOW(border), pic ? pic->getPixbuf() : NULL);
 }
 
 bool gMainWindow::topOnly()
 {
-	if (parent()) return false;
+	if (!isTopLevel()) return false;
 	
 	return top_only;
 }
 
 void gMainWindow::setTopOnly(bool vl)
 {
-	if (parent()) return;
+	if (!isTopLevel()) return;
 
 	gtk_window_set_keep_above (GTK_WINDOW(border),vl);
 	top_only=vl;
@@ -756,6 +779,8 @@ void gMainWindow::drawMask()
 	
 	if (back)
 	{
+		gtk_widget_realize(border);
+		gtk_widget_realize(widget);
 		gdk_window_set_back_pixmap(border->window, back, FALSE);
 		gdk_window_set_back_pixmap(GTK_LAYOUT(widget)->bin_window, back, FALSE);
 		gdk_window_clear(border->window);
@@ -869,6 +894,9 @@ bool gMainWindow::doClose()
 		else
 			opened = false;
 		_closing = false;
+		
+		if (modal())
+			gApplication::exitLoop();
   }
   
   if (!opened) // && !modal())
@@ -956,7 +984,7 @@ int gMainWindow::clientY()
 {
 	GtkRequisition req;
 
-	if (menuBar)
+	if (menuBar && GTK_WIDGET_VISIBLE(GTK_WIDGET(menuBar)))
 	{
 		gtk_widget_size_request(GTK_WIDGET(menuBar), &req);
 		return req.height;
@@ -975,7 +1003,7 @@ int gMainWindow::clientHeight()
 	GtkRequisition req;
 	int h = 0;
 
-	if (menuBar)
+	if (menuBar && GTK_WIDGET_VISIBLE(GTK_WIDGET(menuBar)))
 	{
 		gtk_widget_size_request(GTK_WIDGET(menuBar), &req);
 		h = req.height;
@@ -1015,16 +1043,21 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 {
 	GtkWidget *new_border;
 	int w, h;
+	gColor fg, bg;
 	
 	if (_xembed)
 		return;
 	
-	if (!parent() && newpr)
+	bg = background();
+	fg = foreground();
+	
+	if (isTopLevel() && newpr)
 	{
 		gtk_window_remove_accel_group(GTK_WINDOW(topLevel()->border), accel);
 		
 		new_border = gtk_event_box_new();
 		gtk_widget_reparent(widget, new_border);
+		gMenu::embedMenuBar(this, new_border);
 		_no_delete = true;
 		gtk_widget_destroy(border);
 		_no_delete = false;
@@ -1034,16 +1067,20 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 		connectParent();
 		borderSignals();
 		initWindow();	
+		setBackground(bg);
+		setForeground(fg);
+		setFont(font());
 		
 		move(x, y);
 		gtk_widget_set_size_request(border, width(), height());
 	}
-	else if (parent() && !newpr)
+	else if (!isTopLevel() && !newpr)
 	{
 		gtk_window_remove_accel_group(GTK_WINDOW(topLevel()->border), accel);
 		// TODO: test that
 		new_border = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 		gtk_widget_reparent(widget, new_border);
+		gMenu::embedMenuBar(this, new_border);
 		_no_delete = true;
 		gtk_widget_destroy(border);
 		_no_delete = false;
@@ -1051,9 +1088,13 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 		
 		parent()->remove(this);
 		parent()->arrange();
+
 		setParent(NULL);
 		initWindow();	
 		borderSignals();
+		setBackground(bg);
+		setForeground(fg);
+		setFont(font());
 		
 		move(x, y);
 		w = width();
