@@ -265,6 +265,7 @@ void gMenu::update()
 		}
 		
 		_oldstyle = _style;
+		updateVisible();
 	}
 	
 	if (_style == MENU)
@@ -349,16 +350,44 @@ void gMenu::initialize()
 	_no_update = false;
 	_destroyed = false;
 	_action = false;
+	_visible = false;
 	
 	sizeGroup = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 	
 	menus=g_list_append (menus,(gpointer)this);
 }
 
+void gMenu::embedMenuBar(gMainWindow *par, GtkWidget *border)
+{
+	GtkVBox *box;
+
+	if (par->menuBar)
+	{
+		box=(GtkVBox*)gtk_vbox_new(false,0);
+		g_object_ref(G_OBJECT(par->menuBar));
+		
+		if (gtk_widget_get_parent(GTK_WIDGET(par->menuBar)))
+			gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(GTK_WIDGET(par->menuBar))), GTK_WIDGET(par->menuBar));
+		
+		gtk_box_pack_start(GTK_BOX(box),GTK_WIDGET(par->menuBar),false,true,0);
+
+		g_object_unref(G_OBJECT(par->menuBar));
+		
+		gtk_widget_reparent(par->widget, GTK_WIDGET(box));
+		gtk_container_add(GTK_CONTAINER(border),GTK_WIDGET(box));
+		
+		//gtk_container_add(GTK_CONTAINER(box),par->widget);
+		gtk_widget_show_all(GTK_WIDGET(box));
+		//g_object_unref(par->widget);
+		gtk_widget_show_all(GTK_WIDGET(par->menuBar));
+		set_gdk_fg_color(GTK_WIDGET(par->menuBar),par->foreground());
+		set_gdk_bg_color(GTK_WIDGET(par->menuBar),par->background());
+	}
+}
+
 gMenu::gMenu(gMainWindow *par,bool hidden)
 {
 	//gControl *ctTop;
-	GtkVBox *box;
 
 	pr = (gpointer)par;
   initialize();
@@ -376,22 +405,13 @@ gMenu::gMenu(gMainWindow *par,bool hidden)
 	{
 		//g_object_ref(par->widget);
 		//gtk_container_remove(GTK_CONTAINER(par->border),par->widget);
-		box=(GtkVBox*)gtk_vbox_new(false,0);
 		par->menuBar=(GtkMenuBar*)gtk_menu_bar_new();
-		gtk_box_pack_start(GTK_BOX(box),GTK_WIDGET(par->menuBar),false,true,0);
-		gtk_widget_reparent(par->widget,GTK_WIDGET(box));
-		gtk_container_add(GTK_CONTAINER(par->border),GTK_WIDGET(box));
-		
-		//gtk_container_add(GTK_CONTAINER(box),par->widget);
-		gtk_widget_show_all(GTK_WIDGET(box));
-		//g_object_unref(par->widget);
-		gtk_widget_show_all(GTK_WIDGET(par->menuBar));
-		set_gdk_fg_color(GTK_WIDGET(par->menuBar),par->foreground());
-		set_gdk_bg_color(GTK_WIDGET(par->menuBar),par->background());
+		embedMenuBar(par, par->border);
 	}
 	
 	//update();
-	setText(" ");
+	setText(NULL);
+	setVisible(!hidden);
 }
 
 gMenu::gMenu(gMenu *par, bool hidden)
@@ -410,6 +430,7 @@ gMenu::gMenu(gMenu *par, bool hidden)
 	//_style = SEPARATOR;
 	//update();
 	setText(NULL);
+	setVisible(!hidden);
 }
 
 gMenu::~gMenu()
@@ -497,41 +518,50 @@ void gMenu::setText(const char *text)
 bool gMenu::isVisible()
 {
 	if (!menu) return false;
-	return GTK_WIDGET_VISIBLE(menu);	
+	return _visible;	
 }
 
-void gMenu::setVisible(bool vl)
+void gMenu::updateVisible()
 {
 	GtkContainer *par;
 	GList *chd,*iter;
-
-	if (!menu) return;
+	bool vl = _visible;
+	
+	if (top_level && _style != MENU)
+		vl = false;
+	
 	g_object_set(G_OBJECT(menu),"visible",vl,(void *)NULL);
 	
 	if (top_level && pr)
 	{
-		par=GTK_CONTAINER(((gMainWindow*)pr)->menuBar);
-		if (vl==true)
-		{
-			g_object_set(G_OBJECT(par),"visible",vl,(void *)NULL);
-			return;
-		}
-		chd=gtk_container_get_children(par);
-		iter=g_list_first(chd);
+		vl = false;
+		par = GTK_CONTAINER(((gMainWindow*)pr)->menuBar);
+		chd = gtk_container_get_children(par);
+		iter = g_list_first(chd);
 		while (iter)
 		{
-			if ( GTK_WIDGET_VISIBLE (GTK_WIDGET(iter->data)) )
+			if (GTK_WIDGET_VISIBLE(GTK_WIDGET(iter->data)))
 			{
-				g_list_free(chd);
-				return;
+				vl = true;
+				break;
 			}
-			iter=g_list_next(iter);
+			iter = g_list_next(iter);
 		}
+		
 		g_list_free(chd);
-		g_object_set(G_OBJECT(par),"visible",vl,(void *)NULL);
+		g_object_set(G_OBJECT(par), "visible", vl, (void *)NULL);
 	}
 }
 
+void gMenu::setVisible(bool vl)
+{
+	if (!menu) return;
+	if (vl == _visible) return;
+	
+	_visible = vl;
+	updateVisible();
+}
+
 void gMenu::setPicture(gPicture *pic)
 {
 	//fprintf(stderr, "gMenu::setPicture: %p\n", pic);
