@@ -70,6 +70,8 @@ public:
 	void initialize();
 	gMenuStyle style() { return _style; }
   void hideSeparators();
+
+	static void embedMenuBar(gMainWindow *win, GtkWidget *border);
 	
 private:
 
@@ -84,8 +86,10 @@ private:
 	unsigned _destroyed : 1;
 	unsigned top_level : 1;
 	unsigned _action : 1;
+	unsigned _visible : 1;
   
   void update();
+  void updateVisible();
 };
 
 #endif
