@@ -256,12 +256,15 @@ static gboolean sg_scroll(GtkWidget *widget,GdkEventScroll *event,gControl *data
 	return false;
 }
 
-static gboolean sg_focus_In(GtkWidget *widget,GdkEventFocus *event,gControl *data)
+gboolean gcb_focus_in(GtkWidget *widget,GdkEventFocus *event,gControl *data)
 {	
 	//fprintf(stderr, "sg_focus_in: %s\n", data->name());
 
 	if (!gApplication::allEvents()) return false;
 
+	if (gDesktop::activeControl() == data)
+		return false;
+
 	gMainWindow::setActiveWindow(data);
 	gDesktop::setActiveControl(data);
 	gKey::setActiveControl(data);
@@ -271,12 +274,15 @@ static gboolean sg_focus_In(GtkWidget *widget,GdkEventFocus *event,gControl *dat
 	return false;
 }
 
-static gboolean sg_focus_Out(GtkWidget *widget,GdkEventFocus *event,gControl *data)
+gboolean gcb_focus_out(GtkWidget *widget,GdkEventFocus *event,gControl *data)
 {	
 	//fprintf(stderr, "sg_focus_out: %s\n", data->name());
 	
 	if (!gApplication::allEvents()) return false;
 	
+	if (gDesktop::activeControl() != data)
+		return false;
+
 	if (data->onFocusEvent) data->onFocusEvent(data,gEvent_FocusOut);
 	
 	gKey::setActiveControl(NULL);
@@ -473,6 +479,9 @@ void gControl::borderSignals()
 		g_signal_connect(G_OBJECT(border),"popup-menu",G_CALLBACK(sg_menu),(gpointer)this);	
 		g_signal_connect_after(G_OBJECT(border),"motion-notify-event",G_CALLBACK(sg_motion),(gpointer)this);
 		g_signal_connect(G_OBJECT(border),"scroll-event",G_CALLBACK(sg_scroll),(gpointer)this);
+		
+		g_signal_connect(G_OBJECT(border),"focus-in-event",G_CALLBACK(gcb_focus_in),(gpointer)this);
+		g_signal_connect(G_OBJECT(border),"focus-out-event",G_CALLBACK(gcb_focus_out),(gpointer)this);
 	}
 }
 
@@ -492,8 +501,8 @@ void gControl::widgetSignals()
 	
 	g_signal_connect(G_OBJECT(widget),"key-press-event",G_CALLBACK(gcb_keypress),(gpointer)this);
 	g_signal_connect(G_OBJECT(widget),"key-release-event",G_CALLBACK(gcb_keyrelease),(gpointer)this);
-	g_signal_connect(G_OBJECT(widget),"focus-in-event",G_CALLBACK(sg_focus_In),(gpointer)this);
-	g_signal_connect(G_OBJECT(widget),"focus-out-event",G_CALLBACK(sg_focus_Out),(gpointer)this);
+	g_signal_connect(G_OBJECT(widget),"focus-in-event",G_CALLBACK(gcb_focus_in),(gpointer)this);
+	g_signal_connect(G_OBJECT(widget),"focus-out-event",G_CALLBACK(gcb_focus_out),(gpointer)this);
 	g_signal_connect(G_OBJECT(widget),"event",G_CALLBACK(sg_event),(gpointer)this);
 }
 
