@@ -50,6 +50,8 @@ gboolean gcb_keypress (GtkWidget *widget, GdkEventKey *event, gControl *data);
 gboolean gcb_keyrelease (GtkWidget *widget, GdkEventKey *event, gControl *data);
 gboolean gcb_button_press(GtkWidget *widget,GdkEventButton *event,gControl *data);
 gboolean gcb_button_release(GtkWidget *widget,GdkEventButton *event,gControl *data);
+gboolean gcb_focus_in(GtkWidget *widget,GdkEventFocus *event,gControl *data);
+gboolean gcb_focus_out(GtkWidget *widget,GdkEventFocus *event,gControl *data);
 
 // Where to scroll to ensure that a specific area is visible
 
@@ -106,4 +108,6 @@ void gt_set_cell_renderer_text_from_font(GtkCellRendererText *cell, gFont *font)
 // Initialize a PangoLayout from a font
 
 void gt_set_layout_from_font(PangoLayout *layout, gFont *font);
+void gt_add_layout_from_font(PangoLayout *layout, gFont *font);
 
+#define gt_pango_to_pixel(_pango) (((_pango) + PANGO_SCALE - 1) / PANGO_SCALE)
