@@ -82,10 +82,10 @@ static void my_error(int code,char *error,char *where);
 static void my_quit (void);
 static void my_main(int *argc, char **argv);
 static void my_timer(GB_TIMER *timer,bool on);
-static void my_wait(long duration);
+static void my_wait(int duration);
 static void my_post(void);
 static int my_loop();
-static void my_watch(int fd, int type, void *callback, long param);
+static void my_watch(int fd, int type, void *callback, intptr_t param);
 bool post_Check=false;
 
 
@@ -383,12 +383,12 @@ static int my_loop()
   return 0;
 }
 
-static void my_wait(long duration)
+static void my_wait(int duration)
 {
-	do_iteration(true);
+	do_iteration(true, true);
 }
 
-static void my_watch(int fd, int type, void *callback, long param)
+static void my_watch(int fd, int type, void *callback, intptr_t param)
 {
 	CWatcher::Add(fd,type,callback,param);
 }
@@ -402,7 +402,7 @@ static void my_error(int code,char *error,char *where)
 
 	showstr=g_strconcat("<b>This application has raised an unexpected<br>error and must abort.</b><p>[", scode, "] ", error, ".\n", where, (void *)NULL);
 
-	gMessage::setTitle(GB.Application.Name());
+	gMessage::setTitle(GB.Application.Title());
 	gMessage::showError(showstr,NULL,NULL,NULL);
 
 	g_free(showstr);
@@ -410,7 +410,7 @@ static void my_error(int code,char *error,char *where)
 
 static void my_lang(char *lang,int rtl)
 {
-	long bucle,ct;
+	int bucle, ct;
 	gControl *iter;
 
 	if (rtl==1)
@@ -427,17 +427,17 @@ static void my_lang(char *lang,int rtl)
 	}
 }
 
-int cande=0;
-
-void do_iteration(bool do_not_block)
+void do_iteration(bool do_not_block, bool do_not_sleep)
 {
 	struct timespec mywait;
 
+	gApplication::_loopLevel++;
+
 	if (do_not_block)
 	{
 		if (gtk_events_pending ())
 			gtk_main_iteration_do (false);
-		else
+		else if (!do_not_sleep)
 		{
 			mywait.tv_sec=0;
 			mywait.tv_nsec=100000;
@@ -447,6 +447,8 @@ void do_iteration(bool do_not_block)
 	else	
 		gtk_main_iteration_do(true);
 	
+	gApplication::_loopLevel--;
+
 	if (post_Check) { post_Check=false; GB.CheckPost(); }
 	gControl::cleanRemovedControls();
 }
