@@ -185,6 +185,10 @@ int MyTableItem::alignment() const
 void MyTableItem::paint( QPainter *p, const QColorGroup &cg, const QRect &cr, bool selected )
 {
 	MyTableData *d = data();
+
+	if (!d) // May happen, do not know why!
+		return;
+	
   int w = cr.width();
   int h = cr.height();
   int x = 0;
@@ -212,9 +216,8 @@ void MyTableItem::paint( QPainter *p, const QColorGroup &cg, const QRect &cr, bo
   {
   	x += _padding;
   	y += _padding;
-  	_padding *= 2;
-  	w -= _padding;
-  	h -= _padding;
+  	w -= _padding * 2;
+  	h -= _padding * 2;
   	
   	if (w < 1 || h < 1)
   		return;
@@ -233,9 +236,9 @@ void MyTableItem::paint( QPainter *p, const QColorGroup &cg, const QRect &cr, bo
     }
     else
     {
-      p->drawPixmap(x + 2, y + (h - pix.height() ) / 2, pix);
-      x += pix.width() + 4;
-      w -= pix.width() + 4;
+      p->drawPixmap(x, y + (h - pix.height() ) / 2, pix);
+      x += pix.width() + _padding;
+      w -= pix.width() + _padding;
     }
   }
 
@@ -263,7 +266,6 @@ QSize MyTableItem::sizeHint() const
 {
   QSize strutSize = QApplication::globalStrut();
   QSize s;
-  int x = 0;
   MyTableItem *item = (MyTableItem *)this;
 	MyTableData *d = item->data();
 	int padding;
@@ -272,33 +274,37 @@ QSize MyTableItem::sizeHint() const
   QString t = item->text();
   QFontMetrics fm = d->font ? QFontMetrics(*(d->font->font)) : table()->fontMetrics();
 
+	padding = QMAX(1, d->padding);
+  
   if (!pix.isNull() )
   {
 	  s = pix.size();
-	  s.setWidth( s.width() + 2 );
-	  x = pix.width() + 2;
+	  s.setWidth(s.width());
+	  if (t.length())
+		  s.setWidth(s.width() + padding);
 	}
 
-	padding = QMAX(1, d->padding) * 2;
-  
-  if ( !wordWrap() && t.find( '\n' ) == -1 )
-  {
-  	s = QSize(s.width() + fm.width(t) + 10,
-		          QMAX( s.height(), fm.height() ) ).expandedTo( strutSize );
-	}
-	else
+	if (t.length())
 	{
-		QRect r = fm.boundingRect(0, 0, table()->columnWidth( col() ) - padding, 0,
-							   wordWrap() ? (alignment() | WordBreak) : alignment(),
-							   t );
-		r.setWidth( QMAX( r.width() + 10, table()->columnWidth(col())));
-
-  	s = QSize( r.width(), QMAX( s.height(), r.height() ) ).expandedTo( strutSize );
-  }
+		if ( !wordWrap() && t.find( '\n' ) == -1 )
+		{
+			s = QSize(s.width() + fm.width(t) + 10,
+								QMAX( s.height(), fm.height() ) ).expandedTo( strutSize );
+		}
+		else
+		{
+			QRect r = fm.boundingRect(0, 0, table()->columnWidth( col() ) - padding, 0,
+									wordWrap() ? (alignment() | WordBreak) : alignment(),
+									t );
+			r.setWidth( QMAX( r.width() + 10, table()->columnWidth(col())));
+	
+			s = QSize( r.width(), QMAX( s.height(), r.height() ) ).expandedTo( strutSize );
+		}
+	}
+  
+	s.setWidth(s.width() + padding * 2);
+	s.setHeight(s.height() + padding * 2);
   
-	s.setWidth(s.width() + padding);
-	s.setHeight(s.height() + padding);
-
   return s;
 }
 
@@ -358,6 +364,9 @@ void MyTable::setColumnWidth(int col, int width)
     adjustColumn(col);
   else
     QTable::setColumnWidth(col, width);
+    
+	if (col == (numCols() - 1) && !_updating_last_column)
+		_last_col_width = width; //columnWidth(numCols() - 1);
 }
 
 /*void MyTable::adjustColumn(int col)
@@ -426,6 +435,7 @@ void MyTable::setNumCols(int newCols)
 {
   int i;
   int col = numCols();
+  bool b;
 
   if (newCols < 0)
     return;
@@ -433,24 +443,10 @@ void MyTable::setNumCols(int newCols)
   _cols = newCols;
   _item->invalidate();
 
-  /*
-  if (_items)
-  {
-    delete[] _items;
-    _items = (MyTableItem **)0;
-  }
-
-  if (_cols > 0)
-  {
-    _items = new (MyTableItem *)[_cols];
-    for (i = 0; i < _cols; i++)
-      _items[i] = new MyTableItem(this);
-  }
-  */
-
+  b = signalsBlocked();
+  blockSignals(true);
   QTable::setNumCols(newCols);
-
-	_last_col_width = 0;
+  blockSignals(b);
 
   if (newCols > col)
   {
@@ -464,38 +460,27 @@ void MyTable::setNumCols(int newCols)
   }
 
   clearSelection();
-  emit currentChanged(-1, -1); 
+
+	_last_col_width = columnWidth(numCols() - 1);
+
+	emit currentChanged(-1, -1); 
 }
 
 void MyTable::setNumRows(int newRows)
 {
-  //int i;
-  //int row = numRows();
-
-  //bool r = verticalHeader()->isResizeEnabled();
-
-  //setLeftMargin(fontMetrics().width(QString::number(newRows) + "W"));
-  //verticalHeader()->setResizeEnabled(false);
+	bool b;
 
   if (newRows < 0)
     return;
 
   _rows = newRows;
   _item->invalidate();
+  
+  b = signalsBlocked();
+  blockSignals(true);
   QTable::setNumRows(newRows);
+  blockSignals(false);
 
-  //verticalHeader()->setResizeEnabled(r);
-
-  /*if (newRows > row)
-  {
-    bool upd = verticalHeader()->isUpdatesEnabled();
-    verticalHeader()->setUpdatesEnabled(false);
-
-    for (i = row; i < newRows; i++)
-      verticalHeader()->setLabel(i, QString::number(i + 1));
-
-    verticalHeader()->setUpdatesEnabled(upd);
-  }*/
   clearSelection();
   emit currentChanged(-1, -1); 
 }
@@ -1636,7 +1621,7 @@ END_PROPERTY
 BEGIN_PROPERTY(CGRIDVIEW_client_height)
 
   WIDGET->updateScrollBars();
-  GB.ReturnInteger(WIDGET->clipper()->height());
+  GB.ReturnInteger(WIDGET->visibleHeight());
 
 END_PROPERTY
 
