@@ -54,6 +54,8 @@
 
 CWIDGET *CWIDGET_destroy_list = 0;
 CWIDGET *CWIDGET_destroy_last = 0;
+CWIDGET *CWIDGET_destroy_current = 0;
+CWIDGET *CWIDGET_active_control = 0;
 
 GB_CLASS CLASS_Control;
 GB_CLASS CLASS_Container;
@@ -215,8 +217,6 @@ void CWIDGET_new(QWidget *w, void *_object, bool no_show, bool no_filter, bool n
 {
   CWidget::add(w, _object, no_filter);
 
-  //qDebug("CWIDGET_new: %p: %p", object, w);
-
   THIS->widget = w;
   THIS->level = MAIN_loop_level;
   THIS->next = 0;
@@ -242,13 +242,13 @@ void CWIDGET_new(QWidget *w, void *_object, bool no_show, bool no_filter, bool n
 		w->show();
 	}
 
-	//WIDGET->setName(THIS->name);
+  //qDebug("CWIDGET_new: (%s %p %s)", GB.GetClassName(THIS), THIS, THIS->name);
 }
 
 
 int CCONTROL_check(void *object)
 {
-  return QWIDGET(object) == NULL;
+  return QWIDGET(object) == NULL || CWIDGET_test_flag(object, WF_DELETED);
 }
 
 
@@ -266,7 +266,7 @@ void CWIDGET_destroy(CWIDGET *object)
   if (CWIDGET_test_flag(object, WF_DELETED))
     return;
 
-  //qDebug("CWIDGET_destroy: %p (%p) :%p:%ld", object, object->widget, object->ob.klass, object->ob.ref);
+  //qDebug("CWIDGET_destroy: (%s %s) ref: %d", GB.GetClassName(object), object->name, object->ob.ref);
 
   if (!CWIDGET_destroy_list)
     CWIDGET_destroy_list = object;
@@ -284,6 +284,7 @@ void CWIDGET_destroy(CWIDGET *object)
 
   object->widget->hide();
   //GB.Ref(object);
+  //CWidget::remove(object->widget);
 }
 
 
@@ -800,9 +801,13 @@ BEGIN_METHOD_VOID(CCONTROL_set_focus)
   CWINDOW *win = CWidget::getWindow(THIS);
 
   if (QWIDGET(win)->isVisible())
+  {
+  	//qDebug("setFocus now (%s %p)", GB.GetClassName(THIS), THIS);
     WIDGET->setFocus();
+  }
   else if ((CWIDGET *)win != THIS)
   {
+  	//qDebug("setFocus later (%s %p)", GB.GetClassName(THIS), THIS);
     GB.Unref(POINTER(&win->focus));
     win->focus = THIS;
     GB.Ref(THIS);
@@ -951,7 +956,7 @@ END_PROPERTY
 
 void CWIDGET_set_color(CWIDGET *_object, int bg, int fg)
 {
-	//qDebug("set_color: (%s %p) bg = %06X fg = %06X", GB.GetClassName(THIS), THIS, bg, fg);
+	//qDebug("set_color: (%s %p %s) bg = %06X fg = %06X", GB.GetClassName(THIS), THIS, THIS->name, bg, fg);
 	//if (bg == COLOR_DEFAULT && fg == COLOR_DEFAULT)
 	WIDGET->unsetPalette();
   	
@@ -963,10 +968,12 @@ void CWIDGET_set_color(CWIDGET *_object, int bg, int fg)
 	
 	THIS->flag.default_bg = bg == COLOR_DEFAULT;
   THIS->flag.default_fg = fg == COLOR_DEFAULT;
+  //qDebug("-> %d %06X %d %06X", THIS->flag.default_bg, WIDGET->paletteBackgroundColor().rgb() & 0xFFFFFF, THIS->flag.default_fg, WIDGET->paletteForegroundColor().rgb() & 0xFFFFFF);
 }
 
 int CWIDGET_get_background(CWIDGET *_object)
 {
+	//qDebug("get_background: (%s %p %s) %d %06X", GB.GetClassName(THIS), THIS, THIS->name, THIS->flag.default_bg, WIDGET->paletteBackgroundColor().rgb() & 0xFFFFFF);
 	if (THIS->flag.default_bg)
 		return COLOR_DEFAULT;
 	else
@@ -975,12 +982,19 @@ int CWIDGET_get_background(CWIDGET *_object)
 	
 int CWIDGET_get_foreground(CWIDGET *_object)
 {
+	//qDebug("get_foreground: (%s %p %s) %d %06X", GB.GetClassName(THIS), THIS, THIS->name, THIS->flag.default_fg, WIDGET->paletteForegroundColor().rgb() & 0xFFFFFF);
 	if (THIS->flag.default_fg)
 		return COLOR_DEFAULT;
 	else
 		return WIDGET->paletteForegroundColor().rgb() & 0xFFFFFF;
 }
 	
+void CWIDGET_get_color(CWIDGET *_object, int *bg, int *fg)
+{
+	*bg = CWIDGET_get_background(THIS);
+	*fg = CWIDGET_get_foreground(THIS);
+}
+	
 BEGIN_PROPERTY(CCONTROL_background)
 
   if (READ_PROPERTY)
@@ -1366,7 +1380,7 @@ CWIDGET *CWidget::get(QObject *o)
   while (o)
   {
     ob = dict[o];
-    if (ob != NULL)
+		if (ob)
       return ob;
 
     o = o->parent();
@@ -1376,15 +1390,9 @@ CWIDGET *CWidget::get(QObject *o)
   return NULL;
 }
 
-CWIDGET *CWidget::getReal(QObject *o)
-{
-  return dict[o];
-}
-
-
 CWIDGET *CWidget::getDesign(QObject *o)
 {
-  CWIDGET *ob;
+  CWIDGET *ob = NULL;
 
 	if (!o->isWidgetType())
 		return NULL;
@@ -1405,18 +1413,25 @@ CWIDGET *CWidget::getDesign(QObject *o)
   	return NULL;
 
  	if (!CWIDGET_test_flag(ob, WF_DESIGN))
-	  return ob;
+ 		goto __FIND;
 
   while (o)
   {
     ob = dict[o];
     if (ob && CWIDGET_test_flag(ob, WF_DESIGN_LEADER))
-    	return ob;
+    	goto __FIND;
 
     o = o->parent();
   }
-
+  
   return NULL;
+
+__FIND:
+
+	if (ob)
+		return ob;
+	else
+		return NULL;
 }
 
 /*
@@ -1551,11 +1566,12 @@ void CWidget::destroy()
 {
   QWidget *w = (QWidget *)sender();
   CWIDGET *ob = CWidget::get(w);
-  QEvent e(EVENT_DESTROY);
 
-  if (ob == NULL)
+  if (!ob)
     return;
 
+  QEvent e(EVENT_DESTROY);
+  
   //qDebug(">> CWidget::destroy %p (%p) :%p:%ld", ob, ob->widget, ob->ob.klass, ob->ob.ref);
 
   //w->blockSignals(TRUE);
@@ -1572,6 +1588,9 @@ void CWidget::destroy()
 
   if (enter == ob)
     enter = NULL;
+	
+	if (CWIDGET_active_control == ob)
+		CWIDGET_active_control = NULL;
 
 	set_name(ob, 0);
 
@@ -1613,7 +1632,6 @@ static void post_lostfocus_event(void *control)
   GB.Unref(&control);
 }
 
-
 bool CWidget::eventFilter(QObject *widget, QEvent *event)
 {
   CWIDGET *control;
@@ -1652,10 +1670,14 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
   {
   	if (GB.CanRaise(control, EVENT_GotFocus))
   	{
-			GB.Ref(control);
-			GB.Post((void (*)())post_gotfocus_event, (intptr_t)control);
+			if (control != CWIDGET_active_control)
+			{
+				CWIDGET_active_control = control;
+				GB.Ref(control);
+				GB.Post((void (*)())post_gotfocus_event, (intptr_t)control);
+			}
 		}
-
+		
 		CWINDOW_activate(control);
     //GB.Raise(control, EVENT_GotFocus, 0);
   }
@@ -1663,8 +1685,12 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
   {
   	if (GB.CanRaise(control, EVENT_LostFocus))
   	{
-			GB.Ref(control);
-			GB.Post((void (*)())post_lostfocus_event, (intptr_t)control);
+			if (control == CWIDGET_active_control)
+			{
+				CWIDGET_active_control = NULL;
+				GB.Ref(control);
+				GB.Post((void (*)())post_lostfocus_event, (intptr_t)control);
+			}
 		}
     //GB.Raise(control, EVENT_LostFocus, 0);
   }
@@ -1811,17 +1837,18 @@ bool CWidget::eventFilter(QObject *widget, QEvent *event)
   {
     QKeyEvent *kevent = (QKeyEvent *)event;
 
+// 		qDebug("QKeyEvent: %s (%s %s) (%s %s) %s%sTL:%d -> %d %s",
+// 			type == QEvent::KeyPress ? "KeyPress" : "KeyRelease",
+// 			widget->className(), widget->name(), GB.GetClassName(control), control->name,
+// 			real ? "R " : "", original ? "O " : "",
+// 			((QWidget *)widget)->isTopLevel(),  
+// 			kevent->key(), (char *)kevent->text().latin1());
+
     #if QT_VERSION <= 0x030005
     if (!real || !original)
       goto _DESIGN;
     #endif
 
-// 		qDebug("QKeyEvent: %s (%s %p) (%s %p) TL:%d -> %d %s",
-// 			type == QEvent::KeyPress ? "KeyPress" : "KeyRelease",
-// 			widget->className(), widget, GB.GetClassName(control), control,
-// 			((QWidget *)widget)->isTopLevel(),  
-// 			kevent->key(), (char *)kevent->text().latin1());
-
     event_id = (type == QEvent::KeyRelease) ? EVENT_KeyRelease : EVENT_KeyPress;
 
     #if QT_VERSION > 0x030005
