@@ -170,7 +170,6 @@ public:
 
 void GDocument::init()
 {
-  maxLength = 0;
   selector = NULL;
   baptismLimit = 0;
 }
@@ -178,7 +177,6 @@ void GDocument::init()
 GDocument::GDocument()
 {
   oldCount = 0;
-  oldMaxLength = 0;
   blockUndo = false;
   tabWidth = 2;
   readOnly = false;
@@ -294,6 +292,23 @@ int GDocument::getIndent(int y, bool *empty)
   return i;
 }
 
+void GDocument::updateLineWidth(int y)
+{
+	FOR_EACH_VIEW(v)
+	{
+		v->updateWidth(y);
+	}
+}
+
+void GDocument::insertLine(int y)
+{
+	lines.insert(y, new GLine());
+	lines.at(y)->modified = lines.at(y)->changed = true;
+	if (baptismLimit > y)
+		baptismLimit++;
+	FOR_EACH_VIEW(v) { v->lineInserted(y); }
+}
+
 void GDocument::insert(int y, int x, const GString & text)
 {
   int pos = 0;
@@ -320,8 +335,7 @@ void GDocument::insert(int y, int x, const GString & text)
 	while (y >= (int)lines.count())
 	{
 		yy = (int)lines.count();
-    lines.insert(yy, new GLine());
-    lines.at(yy)->modified = lines.at(yy)->changed = true;
+		insertLine(yy);
 	}
 
   for(;;)
@@ -337,7 +351,8 @@ void GDocument::insert(int y, int x, const GString & text)
       l->s.insert(x, text.mid(pos, pos2 - pos));
       l->modified = l->changed = true;
 
-      maxLength = GMAX(maxLength, (int)l->s.length());
+      //maxLength = GMAX(maxLength, (int)l->s.length());
+      updateLineWidth(y);
 
       FOR_EACH_VIEW(v)
       {
@@ -359,6 +374,7 @@ void GDocument::insert(int y, int x, const GString & text)
 
       l->s.remove(x, rest.length());
       l->modified = l->changed = true;
+      updateLineWidth(y);
     }
 
     FOR_EACH_VIEW(v)
@@ -368,12 +384,9 @@ void GDocument::insert(int y, int x, const GString & text)
     }
 
     y++;
-
-    lines.insert(y, new GLine());
-    lines.at(y)->modified = lines.at(y)->changed = true;
-    if (baptismLimit > y)
-    	baptismLimit++;
-
+		
+		insertLine(y);
+    
     n = -1;
     x = 0;
 
@@ -384,7 +397,8 @@ void GDocument::insert(int y, int x, const GString & text)
     l->s.insert(x, rest);
     l->modified = l->changed = true;
 
-    maxLength = GMAX(maxLength, (int)l->s.length());
+    //maxLength = GMAX(maxLength, (int)l->s.length());
+    updateLineWidth(y);
   }
 
   updateViews(ys, n);
@@ -402,6 +416,14 @@ void GDocument::insert(int y, int x, const GString & text)
 	}
 }
 
+void GDocument::removeLine(int y)
+{
+	lines.remove(y);
+	if (baptismLimit > y)
+		baptismLimit--;
+	FOR_EACH_VIEW(v) { v->lineRemoved(y); }
+}
+
 void GDocument::remove(int y1, int x1, int y2, int x2)
 {
   GLine *l;
@@ -414,6 +436,12 @@ void GDocument::remove(int y1, int x1, int y2, int x2)
   if (readOnly)
     return;
 
+	FOR_EACH_VIEW(v)
+	{
+		v->nx = v->x;
+		v->ny = v->y;
+	}
+
   l = lines.at(y1);
 
   if (y1 == y2)
@@ -424,11 +452,12 @@ void GDocument::remove(int y1, int x1, int y2, int x2)
 
       l->s.remove(x1, x2 - x1);
       l->modified = l->changed = true;
+      updateLineWidth(y1);
 
       FOR_EACH_VIEW(v)
       {
-        if (v->y == y1 && v->x > x1)
-          v->x = GMAX(x1, v->x - (x2 - x1));
+        if (v->ny == y1 && v->nx > x1)
+          v->nx = GMAX(x1, v->nx - (x2 - x1));
       }
 
       updateViews(y1);
@@ -443,7 +472,8 @@ void GDocument::remove(int y1, int x1, int y2, int x2)
     l->modified = l->changed = true;
     l->state = 0; // force highlighting of next line.
 
-    maxLength = GMAX(maxLength, (int)l->s.length());
+    //maxLength = GMAX(maxLength, (int)l->s.length());
+    updateLineWidth(y1);
 
     for (y = y1 + 1; y < y2; y++)
       text += lines.at(y)->s + '\n';
@@ -451,23 +481,19 @@ void GDocument::remove(int y1, int x1, int y2, int x2)
 
     for (y = y1 + 1; y <= y2; y++)
     {
-      lines.remove(y1 + 1);
-
-			if (baptismLimit > (y1 + 1))
-				baptismLimit--;
+    	removeLine(y1 + 1);
 		}
 
-
     FOR_EACH_VIEW(v)
     {
-      if (v->y > y1)
+      if (v->ny > y1)
       {
-        v->y = GMAX(y1, v->y - (y2 - y1));
-        if (v->y == y1)
-          v->x = x1;
+        v->ny = GMAX(y1, v->ny - (y2 - y1));
+        if (v->ny == y1)
+          v->nx = x1;
       }
-      else if (v->y == y1 && v->x > x1)
-        v->x = x1;
+      else if (v->ny == y1 && v->nx > x1)
+        v->nx = x1;
     }
 
     updateViews(y1, -1);
@@ -476,6 +502,11 @@ void GDocument::remove(int y1, int x1, int y2, int x2)
   addUndo(new GDeleteCommand(y1, x1, y2, x2, text));
 
   emitTextChanged();
+
+	FOR_EACH_VIEW(v)
+	{
+		v->cursorGoto(v->ny, v->nx, FALSE);
+	}
 }
 
 void GDocument::setText(const GString & text)
@@ -484,7 +515,7 @@ void GDocument::setText(const GString & text)
 
   readOnly = false;
   blockUndo = true;
-
+  
   clear();
   insert(0, 0, text);
   colorize(0);
@@ -524,16 +555,16 @@ void GDocument::updateViews(int row, int count)
     FOR_EACH_VIEW(v)
     {
       v->setNumRows(oldCount);
-      v->updateLength();
+      v->updateHeight();
     }
   }
 
-  if (maxLength != oldMaxLength)
+  /*if (maxLength != oldMaxLength)
   {
     oldMaxLength = maxLength;
     FOR_EACH_VIEW(v)
       v->updateLength();
-  }
+  }*/
 
   if (row < 0)
   {
@@ -551,9 +582,6 @@ void GDocument::updateViews(int row, int count)
   {
     for (i = row; i < (uint)(row + count); i++)
     {
-      if (i >= lines.count())
-        v->repaintCell(i, 0);
-      else
         v->updateLine(i);
     }
   }
@@ -564,7 +592,7 @@ void GDocument::updateViews(int row, int count)
     FOR_EACH_VIEW(v)
     {
       v->setNumRows(oldCount);
-      v->updateLength();
+      v->updateHeight();
     }
   }
 
@@ -658,6 +686,7 @@ void GDocument::clearUndo()
 {
   undoList.clear();
   redoList.clear();
+  undoLevel = 0;
 }
 
 void GDocument::addUndo(GCommand *c)
@@ -689,12 +718,18 @@ void GDocument::addUndo(GCommand *c)
 
 void GDocument::begin()
 {
+	if (undoLevel == 0)
+		textHasChanged = false;
+	undoLevel++;
   addUndo(new GBeginCommand());
 }
 
 void GDocument::end()
 {
+	undoLevel--;
   addUndo(new GEndCommand());
+  if (undoLevel == 0 && textHasChanged)
+  	emitTextChanged();
 }
 
 void GDocument::addRedo(GCommand * c)
@@ -709,7 +744,7 @@ bool GDocument::undo()
 {
   int nest;
 
-  if (undoList.isEmpty() || isReadOnly())
+  if (undoList.isEmpty() || isReadOnly() || blockUndo)
     return true;
 
   blockUndo = true;
@@ -717,6 +752,8 @@ bool GDocument::undo()
 
   //qDebug("BEGIN UNDO");
 
+	begin();
+
   do
   {
     GCommand *c = undoList.take();
@@ -733,7 +770,9 @@ bool GDocument::undo()
 
   //qDebug("END UNDO");
 
+	end();
   blockUndo = false;
+  
   return false;
 }
 
@@ -741,12 +780,12 @@ bool GDocument::redo()
 {
   int nest;
 
-  if (redoList.isEmpty() || isReadOnly())
+  if (redoList.isEmpty() || isReadOnly() || blockUndo)
     return true;
 
   blockUndo = true;
-
   nest = 0;
+  begin();
 
   do
   {
@@ -763,8 +802,10 @@ bool GDocument::redo()
     undoList.append(c);
   }
   while (nest);
-
+	
+	end();
   blockUndo = false;
+  
   return false;
 }
 
@@ -1014,7 +1055,8 @@ void GDocument::colorize(int y)
   uint state;
   int tag;
   int nupd = 0;
-
+  bool changed = false;
+  
   if (highlightMode == None)
     return;
 
@@ -1051,21 +1093,20 @@ void GDocument::colorize(int y)
 			(*highlightCallback)(views.first(), state, tag, l->s, &l->highlight, proc);
 			l->proc = proc;
 
-			if (l->baptized)
+			if (old != l->s)
 			{
-				if (old != l->s)
-				{
-					begin();
-					addUndo(new GDeleteCommand(y, 0, y, old.length(), old));
-					if (l->s.length())
-						addUndo(new GInsertCommand(y, 0, y, l->s.length(), l->s));
-					end();
-			    
-			    maxLength = GMAX(maxLength, (int)l->s.length());
-				}
+				begin();
+				addUndo(new GDeleteCommand(y, 0, y, old.length(), old));
+				if (l->s.length())
+					addUndo(new GInsertCommand(y, 0, y, l->s.length(), l->s));
+				end();
+				
+				//maxLength = GMAX(maxLength, (int)l->s.length());
+				updateLineWidth(y);
+				//qDebug("colorize: %d has changed: '%s' -> '%s'", y, old.utf8(), l->s.utf8());
+				l->changed = true;
+				changed = true;
 			}
-			else
-				l->baptized = true;
 		}
 		else
 		{
@@ -1089,6 +1130,9 @@ void GDocument::colorize(int y)
 			lines.at(y)->modified = true;
 	}
 
+	if (changed)
+		emitTextChanged();
+	
 	if (nupd >= 1)
 		updateViews(y - nupd + 1, nupd);
 }
@@ -1136,8 +1180,17 @@ void GDocument::setHighlightMode(int mode, GHighlightCallback cb)
 
 void GDocument::emitTextChanged()
 {
+	if (undoLevel > 0)
+	{
+		textHasChanged = true;
+		return;
+	}
+	
+	//qDebug("emitTextChanged");
+	
 	FOR_EACH_VIEW(v)
+	{
 		v->docTextChanged();
+	}
 }
 
-
