@@ -1,7 +1,7 @@
 /****************************************************************************
 ** GEditor meta object code from reading C++ file 'gview.h'
 **
-** Created: Sat Aug 30 09:42:04 2008
+** Created: Tue May 26 20:53:42 2009
 **      by: The Qt MOC ($Id: qt/moc_yacc.cpp   3.3.8   edited Feb 2 14:59 $)
 **
 ** WARNING! All changes made in this file will be lost!
@@ -60,11 +60,17 @@ QMetaObject* GEditor::staticMetaObject()
 	{ "y", &static_QUType_int, 0, QUParameter::In }
     };
     static const QUMethod slot_3 = {"baptizeVisible", 2, param_slot_3 };
+    static const QUMethod slot_4 = {"unflash", 0, 0 };
+    static const QUMethod slot_5 = {"docTextChangedLater", 0, 0 };
+    static const QUMethod slot_6 = {"ensureCursorVisible", 0, 0 };
     static const QMetaData slot_tbl[] = {
 	{ "blinkTimerTimeout()", &slot_0, QMetaData::Private },
 	{ "scrollTimerTimeout()", &slot_1, QMetaData::Private },
 	{ "baptizeVisible()", &slot_2, QMetaData::Private },
-	{ "baptizeVisible(int,int)", &slot_3, QMetaData::Private }
+	{ "baptizeVisible(int,int)", &slot_3, QMetaData::Private },
+	{ "unflash()", &slot_4, QMetaData::Private },
+	{ "docTextChangedLater()", &slot_5, QMetaData::Private },
+	{ "ensureCursorVisible()", &slot_6, QMetaData::Private }
     };
     static const QUMethod signal_0 = {"cursorMoved", 0, 0 };
     static const QUMethod signal_1 = {"textChanged", 0, 0 };
@@ -84,7 +90,7 @@ QMetaObject* GEditor::staticMetaObject()
     };
     metaObj = QMetaObject::new_metaobject(
 	"GEditor", parentObject,
-	slot_tbl, 4,
+	slot_tbl, 7,
 	signal_tbl, 4,
 #ifndef QT_NO_PROPERTIES
 	0, 0,
@@ -133,6 +139,9 @@ bool GEditor::qt_invoke( int _id, QUObject* _o )
     case 1: scrollTimerTimeout(); break;
     case 2: baptizeVisible(); break;
     case 3: baptizeVisible((int)static_QUType_int.get(_o+1),(int)static_QUType_int.get(_o+2)); break;
+    case 4: unflash(); break;
+    case 5: docTextChangedLater(); break;
+    case 6: ensureCursorVisible(); break;
     default:
 	return QGridView::qt_invoke( _id, _o );
     }
