@@ -13,11 +13,8 @@ GB_DESC CCheckBoxDesc[] =
 
   GB_PROPERTY("Value", "i", CCHECKBOX_value),
   GB_PROPERTY("Tristate", "b", CCHECKBOX_tristate),
-  GB_PROPERTY("Action", "s", CCONTROL_action),
 
-  GB_CONSTANT("_Properties", "s", "*,Action,Text,Tristate,Value{CheckBox.False;True;None}"),
-  GB_CONSTANT("_DefaultEvent", "s", "Click"),
-  GB_CONSTANT("_DefaultSize", "s", "24,3"),
+	CHECKBOX_DESCRIPTION,
 
   GB_EVENT("Click", NULL, NULL, &EVENT_Click),
 
