@@ -143,9 +143,10 @@ void MyTabWidget::setEnabled(bool e)
 void MyTabWidget::forceLayout()
 {
 	bool b = isVisible();
+	QShowEvent e;
 		
 	setWState(WState_Visible);
-	qApp->sendEvent(this, new QShowEvent());
+	qApp->sendEvent(this, &e);
 	if (!b)
 		clearWState(WState_Visible);
 	//qDebug("Y = %d", tabBar()->height());
@@ -203,11 +204,11 @@ static bool remove_page(void *_object, int i)
 	return false;
 }
 
-static bool set_tab_count(void *_object, long new_count)
+static bool set_tab_count(void *_object, int new_count)
 {
-  long count = THIS->stack->count();
-  long i;
-  long index;
+  int count = THIS->stack->count();
+  int i;
+  int index;
   QString label;
   CTab *tab;
 
@@ -226,7 +227,7 @@ static bool set_tab_count(void *_object, long new_count)
     {
     	tab = new CTab(THIS, new MyContainer(WIDGET));
 
-      label.sprintf("Tab %ld", i);
+      label.sprintf("Tab %d", i);
       WIDGET->insertTab(tab->widget, label);
       
       THIS->stack->append(tab);
@@ -274,18 +275,17 @@ BEGIN_METHOD(CTABSTRIP_new, GB_OBJECT parent)
 
   MyTabWidget *wid = new MyTabWidget(QCONTAINER(VARG(parent))); //, 0, Qt::WNoMousePropagation);
 
-  CWIDGET_new(wid, (void *)_object, "TabStrip");
+  QObject::connect(wid, SIGNAL(currentChanged(QWidget *)), &CTabStrip::manager, SLOT(currentChanged(QWidget *)));
 
   THIS->container = NULL;
   THIS->index = -1;
   THIS->stack = new QPtrList<CTab>;
   THIS->stack->setAutoDelete(true);
 
-  QObject::connect(wid, SIGNAL(currentChanged(QWidget *)), &CTabStrip::manager, SLOT(currentChanged(QWidget *)));
-
+  CWIDGET_new(wid, (void *)_object, true);
   set_tab_count(THIS, 1);
-
   wid->show();
+
   //wid->updateLayout();
 
 END_METHOD
@@ -308,9 +308,9 @@ BEGIN_PROPERTY(CTABSTRIP_count)
 END_PROPERTY
 
 
-static bool check_index(CTABSTRIP *_object, long index)
+static bool check_index(CTABSTRIP *_object, int index)
 {
-  if (index < 0 || index >= (long)THIS->stack->count())
+  if (index < 0 || index >= (int)THIS->stack->count())
   {
     GB.Error("Bad index");
     return true;
@@ -341,7 +341,7 @@ BEGIN_PROPERTY(CTABSTRIP_index)
 	}
   else
   {
-    long index = VPROP(GB_INTEGER);
+    int index = VPROP(GB_INTEGER);
 
     if (check_index(THIS, index))
       return;
@@ -368,7 +368,7 @@ END_PROPERTY
 
 BEGIN_METHOD(CTABSTRIP_get, GB_INTEGER index)
 
-  long index = VARG(index);
+  int index = VARG(index);
 
   if (check_index(THIS, index))
     return;
@@ -527,11 +527,23 @@ END_METHOD
 BEGIN_PROPERTY(CTAB_count)
 
   QWidget *page = THIS->stack->at(THIS->index)->widget;
+  QObjectList *list = (QObjectList *)page->children();
 
   if (page->children() == NULL)
     GB.ReturnInteger(0);
   else
-    GB.ReturnInteger(page->children()->count());
+  {
+  	int n = 0;
+  	uint i;
+  	
+  	for (i = 0; i < list->count(); i++)
+  	{
+  		if (CWidget::getReal(list->at(i)))
+  			n++;
+  	}
+  	
+    GB.ReturnInteger(n);
+  }
 
 END_PROPERTY
 
