@@ -131,31 +131,34 @@ static void unload_class(CLASS *class)
   {
     #ifdef OS_64BITS
     	
-    	FREE(&class->load->desc, "unload_class");
-    	FREE(&class->load->cst, "unload_class");
-    	FREE(&class->load->class_ref, "unload_class");
-    	FREE(&class->load->unknown, "unload_class");
-    	FREE(&class->load->event, "unload_class");
-    	FREE(&class->load->ext, "unload_class");
-    	FREE(&class->load->local, "unload_class");
-    	FREE(&class->load->array, "unload_class");
-    	
-    	if (class->debug)
+    	if (class->load)
     	{
-    		int i;
-    		FUNCTION *func;
-    		
-				for (i = 0; i < class->load->n_func; i++)
+				FREE(&class->load->desc, "unload_class");
+				FREE(&class->load->cst, "unload_class");
+				FREE(&class->load->class_ref, "unload_class");
+				FREE(&class->load->unknown, "unload_class");
+				FREE(&class->load->event, "unload_class");
+				FREE(&class->load->ext, "unload_class");
+				FREE(&class->load->local, "unload_class");
+				FREE(&class->load->array, "unload_class");
+				
+				if (class->debug)
 				{
-					func = &class->load->func[i];
-					FREE(&func->debug->local, "unload_class");
+					int i;
+					FUNCTION *func;
+					
+					for (i = 0; i < class->load->n_func; i++)
+					{
+						func = &class->load->func[i];
+						FREE(&func->debug->local, "unload_class");
+					}
+					
+					FREE(&class->load->global, "unload_class");
+					FREE(&class->load->debug, "unload_class");
 				}
 				
-				FREE(&class->load->global, "unload_class");
-				FREE(&class->load->debug, "unload_class");
-    	}
-    	
-    	FREE(&class->load->func, "unload_class");
+				FREE(&class->load->func, "unload_class");
+			}
 
     #endif
     
@@ -336,7 +339,7 @@ CLASS *CLASS_look(const char *name, int len)
   if (TABLE_find_symbol(&_global_table, name, len, (SYMBOL **)(void *)&csym, &index))
   {
     #if DEBUG_COMP
-    fprintf(stderr, " -> %ld in global\n", index);
+    fprintf(stderr, " -> %d in global\n", index);
     #endif
     return csym->class;
   }
@@ -750,9 +753,16 @@ CLASS *CLASS_replace_global(const char *name)
 
     ALLOC(&new_name, len + 2, "CLASS_replace_global");
     snprintf(new_name, len+2, ">%s", name);
+    //*new_name = '>';
+    //strcpy(&new_name[1], name);
+    
     new_class = CLASS_replace_global(new_name);
     FREE(&new_name, "CLASS_replace_global");
 
+		#if DEBUG_COMP
+		fprintf(stderr, "CLASS_replace_global: '%s' -> '%s'\n", name, new_name);
+		#endif
+
     new_name = (char *)new_class->name;
 
     TABLE_find_symbol(&_global_table, name, len, (SYMBOL **)(void *)&csym, NULL);
