@@ -32,7 +32,7 @@
 typedef
   struct {
     CLASS *class;
-    int ref;
+    intptr_t ref;
     }
   OBJECT;
 
@@ -67,8 +67,7 @@ void OBJECT_exit(void);
 void OBJECT_create(void **object, CLASS *class, const char *name, void *parent, int nparam);
 void OBJECT_create_native(void **object, CLASS *class, VALUE *param);
 
-bool OBJECT_is_valid(void *object);
-
+#define OBJECT_is_valid(_object) ((_object) && !(((OBJECT *)_object)->class->check && (*((OBJECT *)_object)->class->check)(_object)))
 #define OBJECT_has_events(_object) ((_object) && (((OBJECT *)_object)->class->n_event != 0))
 
 OBJECT *OBJECT_parent(void *object);
@@ -133,38 +132,18 @@ static INLINE CLASS *OBJECT_class(void *object)
 
 #else /* DEBUG_REF */
 
-/*
 #define OBJECT_ref(_object) \
 { \
   if (_object) \
-    CLASS_ref((void *)(_object)); \
-}
-
-#define OBJECT_unref(_pobject) \
-{ \
-  if (*(_pobject)) \
-    CLASS_unref((void **)(_pobject), TRUE); \
-}
-
-#define OBJECT_unref_keep(_pobject) \
-{ \
-  if (*(_pobject)) \
-    CLASS_unref((void **)(_pobject), FALSE); \
-}
-*/
-
-
-#define OBJECT_ref(_object) \
-{ \
-  if (_object) \
-    ((OBJECT *)(void *)(_object))->ref++; \
+    ((OBJECT *)(_object))->ref++; \
 }
 
+#if 0
 #define OBJECT_unref(_pobject) \
 { \
   if (*(_pobject) && --(*((OBJECT **)(void *)(_pobject)))->ref <= 0) \
   { \
-    CLASS_free((void **)(_pobject)); \
+    CLASS_free((void **)(void *)(_pobject)); \
   } \
 }
 
@@ -173,11 +152,30 @@ static INLINE CLASS *OBJECT_class(void *object)
   if (*(_pobject)) \
     --(*((OBJECT **)(void *)(_pobject)))->ref; \
 }
+#endif
+
+#define OBJECT_unref(_object) \
+{ \
+  if (_object) \
+  { \
+  	if ((--((OBJECT *)(_object))->ref) <= 0) \
+  	{ \
+			CLASS_free(_object); \
+			_object = NULL; \
+		} \
+	} \
+}
+
+#define OBJECT_unref_keep(_object) \
+{ \
+  if (_object) \
+  	--((OBJECT *)(_object))->ref; \
+}
 
 
 #define OBJECT_REF(_ob, _where) OBJECT_ref(_ob)
-#define OBJECT_UNREF(_ob, _where) OBJECT_unref(((void **)(void *)_ob))
-#define OBJECT_UNREF_KEEP(_ob, _where) OBJECT_unref_keep(((void **)(void *)_ob))
+#define OBJECT_UNREF(_ob, _where) OBJECT_unref(_ob)
+#define OBJECT_UNREF_KEEP(_ob, _where) OBJECT_unref_keep(_ob)
 
 #endif /* DEBUG_REF */
 
